// DCTTest.cpp : Defines the entry point for the console application.
//

#include <stdio.h>	// printf() and its ilk
#include <conio.h>	// for kbhit() & getch() to exit main loop
#include <ctype.h>	// for isdigit()
#include <stdlib.h>	// for atoi()

#ifdef _WIN32
#include <windows.h>	// for ::Sleep()
#endif

#include "serial.h"
#include "Processor.h"


// define the settings for the serial ports. The DCT uses 9600/n/8/1,
// and we default to ports 0 and 1 (COM1 and COM2 on Windows)
int port1				= 0;
int bps1				= 9600;
int data1				= 8;
_eSer_Stop stop1		= _ser_stop_1;
_eSer_Parity parity1	= _ser_parity_none;

int port2				= 1;
int bps2				= 9600;
int data2				= 8;
_eSer_Stop stop2		= _ser_stop_1;
_eSer_Parity parity2	= _ser_parity_none;

int showAllData = 0;

void Usage( void );

int isNumericString( const char *str )
	{
	if ( str == NULL )
		{
		return 0;
		}
	for ( ; *str; str++ )
		{
		if ( ! isdigit( *str ) )
			{
			return 0;
			}
		}
	return 1;
	}

int main( int argc, char* argv[] )
	{
	int i;

	fprintf( stderr, "DCTTest v1.0   Copyright (c) 2003 Don Starr\n" );

	// parse the command line

	int index = 1;
	if ( argc == 2 || argc == 4 )
		{
		// just the S param, or S plus the port numbers
		if ( argv[index][0] == 'S' )
			{
			showAllData = 1;
			}
		else
			{
			Usage();
			return -1;
			}
		index ++;
		}
	if ( argc == 3 || argc == 4 )
		{
		// the port numbers, or port numbers plus the S param
		if ( !isNumericString( argv[index] ) ||
			 !isNumericString( argv[index+1] ) )
			{
			Usage();
			return -1;
			}
		port1 = atoi( argv[index] );
		port2 = atoi( argv[index+1] );
		if ( port1 < 0 || port1 > 255 ||
			 port2 < 0 || port2 > 255 )
			{
			Usage();
			return -1;
			}
		}
	else
	if ( argc != 1 )
		{
		// some invalid number of parameters
		Usage();
		return -1;
		}


	fprintf( stdout, "opening serial port %d: %d/%d/%s/%s ...\n",
		port1,
		bps1,
		data1,
		stop1 == _ser_stop_1 ? "1"
			: stop1 == _ser_stop_1_5 ? "1.5"
			: "2",
		parity1 == _ser_parity_none ? "none"
			: parity1 == _ser_parity_even ? "even"
			: "odd" );
	_ser_struct *serport1 = ser_open( port1, bps1, data1, stop1, parity1 );

	fprintf( stdout, "opening serial port %d: %d/%d/%s/%s ...\n",
		port2,
		bps2,
		data2,
		stop2 == _ser_stop_1 ? "1"
			: stop2 == _ser_stop_1_5 ? "1.5"
			: "2",
		parity2 == _ser_parity_none ? "none"
			: parity2 == _ser_parity_even ? "even"
			: "odd" );
	_ser_struct *serport2 = ser_open( port2, bps2, data2, stop2, parity2 );

	int done = 0;

	_sProcessorInfo *pinfo1 = ProcessorInit( showAllData );
	_sProcessorInfo *pinfo2 = ProcessorInit( showAllData );

	if ( serport1 == NULL || serport2 == NULL ||
		 pinfo1 == NULL || pinfo2 == NULL )
		{
		done = 1;
		}
	else
		{
		fprintf( stderr, "press ESC to exit\n\n" );
		}

	fprintf( stdout, "Sidecar (e.g. TiVo)                     DCT\n" );
	fprintf( stdout, "========================================"
		             "=======================================\n" );

	while ( ! done )
		{
		int somethingToDo = 0;
		
		// see if any keyboard keys hit...
		if ( kbhit() )
			{
			somethingToDo = 1;

			if ( getch() == 27 )
				{
				// bail out on the ESC key
				done = 1;
				}
			}

		int avail;
#define _MaxReadSize	20		// to fit on a line if showAllData is set
		unsigned char buffer[_MaxReadSize];

		// purge port1's input
		while ( (avail = ser_peek( serport1 )) != 0 )
			{
			int toRead = __min( avail, _MaxReadSize );
			int read = ser_read( serport1, buffer, toRead, 100 );
			if ( read )
				{
				if ( showAllData )
					{
					fprintf( stdout, "Port %1d: ", port1 );
					for ( i=0; i<read; i++ )
						{
						fprintf( stdout, "%02X ", buffer[i] );
						}
					fprintf( stdout, "\n" );
					}
				for ( i=0; i<read; i++ )
					{
					ProcessByte( pinfo1, buffer[i] );
					}

				somethingToDo = 1;

				// write it out to the other port (we're a 'peeking pass-through')
				int totalWritten = 0;
				while ( totalWritten < read )
					{
					int written = ser_write( serport2, buffer+totalWritten, read-totalWritten, 1000 );
					if ( !written )
						{
						fprintf( stderr, "ERROR: writing to serport2!!!\n" );
						break;
						}
					totalWritten += written;
					}
				}
			else
				{
				// the peek function said something was there, but the read function
				// got nothing. should probably bail out of the program here (or at
				// least purge the port and maybe close/reopen it).
				break;
				}
			}

		// purge port2's input
		while ( (avail = ser_peek( serport2 )) != 0 )
			{
			int toRead = __min( avail, _MaxReadSize );
			int read = ser_read( serport2, buffer, toRead, 100 );
			if ( read )
				{
				if ( showAllData )
					{
					fprintf( stdout, "Port %1d: ", port2 );
					for ( i=0; i<read; i++ )
						{
						fprintf( stdout, "%02X ", buffer[i] );
						}
					fprintf( stdout, "\n" );
					}
				for ( i=0; i<read; i++ )
					{
					ProcessByte( pinfo2, buffer[i] );
					}

				somethingToDo = 1;

				// write it out to the other port (we're a 'peeking pass-through')
				int totalWritten = 0;
				while ( totalWritten < read )
					{
					int written = ser_write( serport1, buffer+totalWritten, read-totalWritten, 1000 );
					if ( !written )
						{
						fprintf( stderr, "ERROR: writing to serport1!!!\n" );
						break;
						}
					totalWritten += written;
					}
				}
			else
				{
				// the peek function said something was there, but the read function
				// got nothing. should probably bail out of the program here (or at
				// least purge the port and maybe close/reopen it).
				break;
				}
			}
				
		// if we didn't either get something from a serial port or get
		// a key, wait a little bit (so we don't hog the CPU).
		if ( ! somethingToDo )
			{
			// replace the following with an appropriate "wait" for your
			// environment. In Win32, this is a "sleep for 5 milliseconds".
#ifdef _WIN32
			::Sleep( 5 );
#endif
			}

		}

	if ( serport1 != NULL )
		{
		ser_close( serport1 );
		}
	if ( serport2 != NULL )
		{
		ser_close( serport2 );
		}

	ProcessorCleanup( pinfo1 );
	ProcessorCleanup( pinfo2 );

	return 0;
	}

void Usage( void )
	{
	fprintf( stderr, "\n" );
	fprintf( stderr, "Usage:  dcttest [S] [port1 port2]\n" );
	fprintf( stderr, "        <S> displays all Rx data, plus extra packet"
	                          "decoding\n" );
	fprintf( stderr, "        <port1> is a decimal index indicating port "
		                      "number (e.g. 0==COM1). RANGE: 0-255. DEFAULT: "
							  "0 (COM1)\n" );
	fprintf( stderr, "        <port2> is a decimal index indicating port "
	                          "number (e.g. 1==COM2). RANGE: 0-255. DEFAULT: "
							  "1 (COM2)\n" );
	fprintf( stderr, "<port1> and <port2> must both be specified, or neither "
	                          "must be specified.\n" );
	fprintf( stderr, "\n" );
	}


